#!/usr/bin/env python
#
# Copyright (C) 2009 Google Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


# This module is used for version 2 of the Google Data APIs.


__author__ = 'j.s@google.com (Jeff Scudder)'


import atom.core


XML_TEMPLATE = '{http://www.w3.org/XML/1998/namespace}%s'
ATOM_TEMPLATE = '{http://www.w3.org/2005/Atom}%s'
APP_TEMPLATE_V1 = '{http://purl.org/atom/app#}%s'
APP_TEMPLATE_V2 = '{http://www.w3.org/2007/app}%s'


class Name(atom.core.XmlElement):
  """The atom:name element."""
  _qname = ATOM_TEMPLATE % 'name'


class Email(atom.core.XmlElement):
  """The atom:email element."""
  _qname = ATOM_TEMPLATE % 'email'


class Uri(atom.core.XmlElement):
  """The atom:uri element."""
  _qname = ATOM_TEMPLATE % 'uri'


class Person(atom.core.XmlElement):
  """A foundation class which atom:author and atom:contributor extend.

  A person contains information like name, email address, and web page URI for
  an author or contributor to an Atom feed.
  """
  name = Name
  email = Email
  uri = Uri


class Author(Person):
  """The atom:author element.

  An author is a required element in Feed unless each Entry contains an Author.
  """
  _qname = ATOM_TEMPLATE % 'author'


class Contributor(Person):
  """The atom:contributor element."""
  _qname = ATOM_TEMPLATE % 'contributor'


class Link(atom.core.XmlElement):
  """The atom:link element."""
  _qname = ATOM_TEMPLATE % 'link'
  href = 'href'
  rel = 'rel'
  type = 'type'
  hreflang = 'hreflang'
  title = 'title'
  length = 'length'


class Generator(atom.core.XmlElement):
  """The atom:generator element."""
  _qname = ATOM_TEMPLATE % 'generator'
  uri = 'uri'
  version = 'version'


class Text(atom.core.XmlElement):
  """A foundation class from which atom:title, summary, etc. extend.

  This class should never be instantiated.
  """
  type = 'type'


class Title(Text):
  """The atom:title element."""
  _qname = ATOM_TEMPLATE % 'title'


class Subtitle(Text):
  """The atom:subtitle element."""
  _qname = ATOM_TEMPLATE % 'subtitle'


class Rights(Text):
  """The atom:rights element."""
  _qname = ATOM_TEMPLATE % 'rights'


class Summary(Text):
  """The atom:summary element."""
  _qname = ATOM_TEMPLATE % 'summary'


class Content(Text):
  """The atom:content element."""
  _qname = ATOM_TEMPLATE % 'content'
  src = 'src'


class Category(atom.core.XmlElement):
  """The atom:category element."""
  _qname = ATOM_TEMPLATE % 'category'
  term = 'term'
  scheme = 'scheme'
  label = 'label'


class Id(atom.core.XmlElement):
  """The atom:id element."""
  _qname = ATOM_TEMPLATE % 'id'


class Icon(atom.core.XmlElement):
  """The atom:icon element."""
  _qname = ATOM_TEMPLATE % 'icon'


class Logo(atom.core.XmlElement):
  """The atom:logo element."""
  _qname = ATOM_TEMPLATE % 'logo'


class Draft(atom.core.XmlElement):
  """The app:draft element which indicates if this entry should be public."""
  _qname = (APP_TEMPLATE_V1 % 'draft', APP_TEMPLATE_V2 % 'draft')


class Control(atom.core.XmlElement):
  """The app:control element indicating restrictions on publication.

  The APP control element may contain a draft element indicating whether or
  not this entry should be publicly available.
  """
  _qname = (APP_TEMPLATE_V1 % 'control', APP_TEMPLATE_V2 % 'control')
  draft = Draft


class Date(atom.core.XmlElement):
  """A parent class for atom:updated, published, etc."""


class Updated(Date):
  """The atom:updated element."""
  _qname = ATOM_TEMPLATE % 'updated'


class Published(Date):
  """The atom:published element."""
  _qname = ATOM_TEMPLATE % 'published'


class LinkFinder(object):
  """An "interface" providing methods to find link elements

  Entry elements often contain multiple links which differ in the rel
  attribute or content type. Often, developers are interested in a specific
  type of link so this class provides methods to find specific classes of
  links.

  This class is used as a mixin in Atom entries and feeds.
  """

  def find_url(self, rel):
    """Returns the URL (as a string) in a link with the desired rel value."""
    for link in self.link:
      if link.rel == rel and link.href:
        return link.href
    return None

  FindUrl = find_url

  def get_link(self, rel):
    """Returns a link object which has the desired rel value.

    If you are interested in the URL instead of the link object,
    consider using find_url instead.
    """
    for link in self.link:
      if link.rel == rel and link.href:
        return link
    return None

  GetLink = get_link

  def find_self_link(self):
    """Find the first link with rel set to 'self'

    Returns:
      A str containing the link's href or None if none of the links had rel
      equal to 'self'
    """
    return self.find_url('self')

  FindSelfLink = find_self_link

  def get_self_link(self):
    return self.get_link('self')

  GetSelfLink = get_self_link

  def find_edit_link(self):
    return self.find_url('edit')

  FindEditLink = find_edit_link

  def get_edit_link(self):
    return self.get_link('edit')

  GetEditLink = get_edit_link

  def find_edit_media_link(self):
    link = self.find_url('edit-media')
    # Search for media-edit as well since Picasa API used media-edit instead.
    if link is None:
      return self.find_url('media-edit')
    return link

  FindEditMediaLink = find_edit_media_link

  def get_edit_media_link(self):
    link = self.get_link('edit-media')
    if link is None:
      return self.get_link('media-edit')
    return link

  GetEditMediaLink = get_edit_media_link

  def find_next_link(self):
    return self.find_url('next')

  FindNextLink = find_next_link

  def get_next_link(self):
    return self.get_link('next')

  GetNextLink = get_next_link

  def find_license_link(self):
    return self.find_url('license')

  FindLicenseLink = find_license_link

  def get_license_link(self):
    return self.get_link('license')

  GetLicenseLink = get_license_link

  def find_alternate_link(self):
    return self.find_url('alternate')

  FindAlternateLink = find_alternate_link

  def get_alternate_link(self):
    return self.get_link('alternate')

  GetAlternateLink = get_alternate_link


class FeedEntryParent(atom.core.XmlElement, LinkFinder):
  """A super class for atom:feed and entry, contains shared attributes"""
  author = [Author]
  category = [Category]
  contributor = [Contributor]
  id = Id
  link = [Link]
  rights = Rights
  title = Title
  updated = Updated

  def __init__(self, atom_id=None, text=None, *args, **kwargs):
    if atom_id is not None:
      self.id = atom_id
    atom.core.XmlElement.__init__(self, text=text, *args, **kwargs)


class Source(FeedEntryParent):
  """The atom:source element."""
  _qname = ATOM_TEMPLATE % 'source'
  generator = Generator
  icon = Icon
  logo = Logo
  subtitle = Subtitle


class Entry(FeedEntryParent):
  """The atom:entry element."""
  _qname = ATOM_TEMPLATE % 'entry'
  content = Content
  published = Published
  source = Source
  summary = Summary
  control = Control


class Feed(Source):
  """The atom:feed element which contains entries."""
  _qname = ATOM_TEMPLATE % 'feed'
  entry = [Entry]


class ExtensionElement(atom.core.XmlElement):
  """Provided for backwards compatibility to the v1 atom.ExtensionElement."""

  def __init__(self, tag=None, namespace=None, attributes=None,
      children=None, text=None, *args, **kwargs):
    if namespace:
      self._qname = '{%s}%s' % (namespace, tag)
    else:
      self._qname = tag
    self.children = children or []
    self.attributes = attributes or {}
    self.text = text

  _BecomeChildElement = atom.core.XmlElement._become_child



#!/usr/bin/env python
#
# Copyright (C) 2009 Google Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from types import ListType, DictionaryType


"""Contains a client to communicate with the Contacts servers.

For documentation on the Contacts API, see:
http://code.google.com/apis/contatcs/
"""

__author__ = 'vinces1979@gmail.com (Vince Spicer)'


import gdata.client
import gdata.contacts.data
import atom.client
import atom.data
import atom.http_core
import gdata.gauth

DEFAULT_BATCH_URL = ('https://www.google.com/m8/feeds/contacts/default/full'
                     '/batch')
DEFAULT_PROFILES_BATCH_URL = ('https://www.google.com/m8/feeds/profiles/domain/'
                              '%s/full/batch')

class ContactsClient(gdata.client.GDClient):
  api_version = '3'
  auth_service = 'cp'
  server = "www.google.com"
  contact_list = "default"
  auth_scopes = gdata.gauth.AUTH_SCOPES['cp']
  ssl = True


  def __init__(self, domain=None, auth_token=None, **kwargs):
    """Constructs a new client for the Email Settings API.

    Args:
      domain: string The Google Apps domain (if any).
      kwargs: The other parameters to pass to the gdata.client.GDClient
          constructor.
    """
    gdata.client.GDClient.__init__(self, auth_token=auth_token, **kwargs)
    self.domain = domain

  def get_feed_uri(self, kind='contacts', contact_list=None, projection='full',
                  scheme="https"):
    """Builds a feed URI.

    Args:
      kind: The type of feed to return, typically 'groups' or 'contacts'.
        Default value: 'contacts'.
      contact_list: The contact list to return a feed for.
        Default value: self.contact_list.
      projection: The projection to apply to the feed contents, for example
        'full', 'base', 'base/12345', 'full/batch'. Default value: 'full'.
      scheme: The URL scheme such as 'http' or 'https', None to return a
          relative URI without hostname.

    Returns:
      A feed URI using the given kind, contact list, and projection.
      Example: '/m8/feeds/contacts/default/full'.
    """
    contact_list = contact_list or self.contact_list
    if kind == 'profiles':
      contact_list = 'domain/%s' % self.domain
    prefix = scheme and '%s://%s' % (scheme, self.server) or ''
    return '%s/m8/feeds/%s/%s/%s' % (prefix, kind, contact_list, projection)

  GetFeedUri = get_feed_uri

  def get_contact(self, uri, desired_class=gdata.contacts.data.ContactEntry,
                  auth_token=None, **kwargs):
    return self.get_entry(uri, auth_token=auth_token,
                          desired_class=desired_class, **kwargs)


  GetContact = get_contact


  def create_contact(self, new_contact, insert_uri=None,  auth_token=None,  **kwargs):
    """Adds an new contact to Google Contacts.

    Args:
      new_contact: atom.Entry or subclass A new contact which is to be added to
                Google Contacts.
      insert_uri: the URL to post new contacts to the feed
      url_params: dict (optional) Additional URL parameters to be included
                  in the insertion request.
      escape_params: boolean (optional) If true, the url_parameters will be
                     escaped before they are included in the request.

    Returns:
      On successful insert,  an entry containing the contact created
      On failure, a RequestError is raised of the form:
        {'status': HTTP status code from server,
         'reason': HTTP reason from the server,
         'body': HTTP body of the server's response}
    """
    insert_uri = insert_uri or self.GetFeedUri()
    return self.Post(new_contact, insert_uri,
                     auth_token=auth_token,  **kwargs)

  CreateContact = create_contact

  def add_contact(self, new_contact, insert_uri=None, auth_token=None,
                  billing_information=None, birthday=None, calendar_link=None, **kwargs):
    """Adds an new contact to Google Contacts.

    Args:
      new_contact: atom.Entry or subclass A new contact which is to be added to
                Google Contacts.
      insert_uri: the URL to post new contacts to the feed
      url_params: dict (optional) Additional URL parameters to be included
                  in the insertion request.
      escape_params: boolean (optional) If true, the url_parameters will be
                     escaped before they are included in the request.

    Returns:
      On successful insert,  an entry containing the contact created
      On failure, a RequestError is raised of the form:
        {'status': HTTP status code from server,
         'reason': HTTP reason from the server,
         'body': HTTP body of the server's response}
    """

    contact = gdata.contacts.data.ContactEntry()

    if billing_information is not None:
      if not isinstance(billing_information, gdata.contacts.data.BillingInformation):
        billing_information = gdata.contacts.data.BillingInformation(text=billing_information)

      contact.billing_information = billing_information

    if birthday is not None:
      if not isinstance(birthday, gdata.contacts.data.Birthday):
        birthday = gdata.contacts.data.Birthday(when=birthday)

      contact.birthday = birthday

    if calendar_link is not None:
      if type(calendar_link) is not ListType:
        calendar_link = [calendar_link]

      for link in calendar_link:
        if not isinstance(link, gdata.contacts.data.CalendarLink):
          if type(link) is not DictionaryType:
            raise TypeError, "calendar_link Requires dictionary not %s" % type(link)

          link = gdata.contacts.data.CalendarLink(
                                                  rel=link.get("rel", None),
                                                  label=link.get("label", None),
                                                  primary=link.get("primary", None),
                                                  href=link.get("href", None),
                                                  )

        contact.calendar_link.append(link)

    insert_uri = insert_uri or self.GetFeedUri()
    return self.Post(contact, insert_uri,
                     auth_token=auth_token,  **kwargs)

  AddContact = add_contact

  def get_contacts(self, uri=None, desired_class=gdata.contacts.data.ContactsFeed,
                   auth_token=None, **kwargs):
    """Obtains a feed with the contacts belonging to the current user.

    Args:
      auth_token: An object which sets the Authorization HTTP header in its
                  modify_request method. Recommended classes include
                  gdata.gauth.ClientLoginToken and gdata.gauth.AuthSubToken
                  among others. Represents the current user. Defaults to None
                  and if None, this method will look for a value in the
                  auth_token member of SpreadsheetsClient.
      desired_class: class descended from atom.core.XmlElement to which a
                     successful response should be converted. If there is no
                     converter function specified (desired_class=None) then the
                     desired_class will be used in calling the
                     atom.core.parse function. If neither
                     the desired_class nor the converter is specified, an
                     HTTP reponse object will be returned. Defaults to
                     gdata.spreadsheets.data.SpreadsheetsFeed.
    """
    uri = uri or self.GetFeedUri()
    return self.get_feed(uri, auth_token=auth_token,
                         desired_class=desired_class, **kwargs)

  GetContacts = get_contacts

  def get_group(self, uri=None, desired_class=gdata.contacts.data.GroupEntry,
                auth_token=None, **kwargs):
    """ Get a single groups details
    Args:
        uri:  the group uri or id
    """
    return self.get_entry(uri, desired_class=desired_class, auth_token=auth_token, **kwargs)

  GetGroup = get_group

  def get_groups(self, uri=None, desired_class=gdata.contacts.data.GroupsFeed,
                 auth_token=None, **kwargs):
    uri = uri or self.GetFeedUri('groups')
    return self.get_feed(uri, desired_class=desired_class, auth_token=auth_token, **kwargs)

  GetGroups = get_groups

  def create_group(self, new_group, insert_uri=None, url_params=None,
                   desired_class=None, **kwargs):
    insert_uri = insert_uri or self.GetFeedUri('groups')
    return self.Post(new_group, insert_uri, url_params=url_params,
                     desired_class=desired_class, **kwargs)

  CreateGroup = create_group

  def update_group(self, edit_uri, updated_group, url_params=None,
                   escape_params=True, desired_class=None, auth_token=None, **kwargs):
    return self.Put(updated_group, self._CleanUri(edit_uri),
                    url_params=url_params,
                    escape_params=escape_params,
                    desired_class=desired_class,
                    auth_token=auth_token, **kwargs)

  UpdateGroup = update_group

  def delete_group(self, group_object, auth_token=None, force=False, **kws):
    return self.Delete(group_object, auth_token=auth_token, force=force, **kws)

  DeleteGroup = delete_group

  def change_photo(self, media, contact_entry_or_url, content_type=None,
                   content_length=None, auth_token=None, **kwargs):
    """Change the photo for the contact by uploading a new photo.

    Performs a PUT against the photo edit URL to send the binary data for the
    photo.

    Args:
      media: filename, file-like-object, or a gdata.data.MediaSource object to send.
      contact_entry_or_url: ContactEntry or str If it is a ContactEntry, this
                            method will search for an edit photo link URL and
                            perform a PUT to the URL.
      content_type: str (optional) the mime type for the photo data. This is
                    necessary if media is a file or file name, but if media
                    is a MediaSource object then the media object can contain
                    the mime type. If media_type is set, it will override the
                    mime type in the media object.
      content_length: int or str (optional) Specifying the content length is
                      only required if media is a file-like object. If media
                      is a filename, the length is determined using
                      os.path.getsize. If media is a MediaSource object, it is
                      assumed that it already contains the content length.
    """
    ifmatch_header = None
    if isinstance(contact_entry_or_url, gdata.contacts.data.ContactEntry):
      photo_link = contact_entry_or_url.GetPhotoLink()
      uri = photo_link.href
      ifmatch_header = atom.client.CustomHeaders(
          **{'if-match': photo_link.etag})
    else:
      uri = contact_entry_or_url
    if isinstance(media, gdata.data.MediaSource):
      payload = media
    # If the media object is a file-like object, then use it as the file
    # handle in the in the MediaSource.
    elif hasattr(media, 'read'):
      payload = gdata.data.MediaSource(file_handle=media,
          content_type=content_type, content_length=content_length)
    # Assume that the media object is a file name.
    else:
      payload = gdata.data.MediaSource(content_type=content_type,
          content_length=content_length, file_path=media)
    return self.Put(uri=uri, data=payload, auth_token=auth_token,
                    ifmatch_header=ifmatch_header, **kwargs)

  ChangePhoto = change_photo

  def get_photo(self, contact_entry_or_url, auth_token=None, **kwargs):
    """Retrives the binary data for the contact's profile photo as a string.

    Args:
      contact_entry_or_url: a gdata.contacts.ContactEntry object or a string
         containing the photo link's URL. If the contact entry does not
         contain a photo link, the image will not be fetched and this method
         will return None.
    """
    # TODO: add the ability to write out the binary image data to a file,
    # reading and writing a chunk at a time to avoid potentially using up
    # large amounts of memory.
    url = None
    if isinstance(contact_entry_or_url, gdata.contacts.data.ContactEntry):
      photo_link = contact_entry_or_url.GetPhotoLink()
      if photo_link:
        url = photo_link.href
    else:
      url = contact_entry_or_url
    if url:
      return self.Get(url, auth_token=auth_token, **kwargs).read()
    else:
      return None

  GetPhoto = get_photo

  def delete_photo(self, contact_entry_or_url, auth_token=None, **kwargs):
    """Delete the contact's profile photo.

    Args:
      contact_entry_or_url: a gdata.contacts.ContactEntry object or a string
         containing the photo link's URL.
    """
    uri = None
    ifmatch_header = None
    if isinstance(contact_entry_or_url, gdata.contacts.data.ContactEntry):
      photo_link = contact_entry_or_url.GetPhotoLink()
      if photo_link.etag:
        uri = photo_link.href
        ifmatch_header = atom.client.CustomHeaders(
            **{'if-match': photo_link.etag})
      else:
        # No etag means no photo has been assigned to this contact.
        return
    else:
      uri = contact_entry_or_url
    if uri:
      self.Delete(entry_or_uri=uri, auth_token=auth_token,
                  ifmatch_header=ifmatch_header, **kwargs)

  DeletePhoto = delete_photo

  def get_profiles_feed(self, uri=None, auth_token=None, **kwargs):
    """Retrieves a feed containing all domain's profiles.

    Args:
      uri: string (optional) the URL to retrieve the profiles feed,
          for example /m8/feeds/profiles/default/full

    Returns:
      On success, a ProfilesFeed containing the profiles.
      On failure, raises a RequestError.
    """

    uri = uri or self.GetFeedUri('profiles')
    return self.get_feed(uri, auth_token=auth_token,
                         desired_class=gdata.contacts.data.ProfilesFeed, **kwargs)

  GetProfilesFeed = get_profiles_feed

  def get_profile(self, uri, auth_token=None, **kwargs):
    """Retrieves a domain's profile for the user.

    Args:
      uri: string the URL to retrieve the profiles feed,
          for example /m8/feeds/profiles/default/full/username

    Returns:
      On success, a ProfileEntry containing the profile for the user.
      On failure, raises a RequestError
    """
    return self.get_entry(uri,
                          desired_class=gdata.contacts.data.ProfileEntry,
                          auth_token=auth_token, **kwargs)

  GetProfile = get_profile

  def update_profile(self, updated_profile, auth_token=None, force=False, **kwargs):
    """Updates an existing profile.

    Args:
      updated_profile: atom.Entry or subclass containing
                       the Atom Entry which will replace the profile which is
                       stored at the edit_url.
      auth_token: An object which sets the Authorization HTTP header in its
                  modify_request method. Recommended classes include
                  gdata.gauth.ClientLoginToken and gdata.gauth.AuthSubToken
                  among others. Represents the current user. Defaults to None
                  and if None, this method will look for a value in the
                  auth_token member of ContactsClient.
      force: boolean stating whether an update should be forced. Defaults to
             False. Normally, if a change has been made since the passed in
             entry was obtained, the server will not overwrite the entry since
             the changes were based on an obsolete version of the entry.
             Setting force to True will cause the update to silently
             overwrite whatever version is present.
      url_params: dict (optional) Additional URL parameters to be included
                  in the insertion request.
      escape_params: boolean (optional) If true, the url_parameters will be
                     escaped before they are included in the request.

    Returns:
      On successful update,  a httplib.HTTPResponse containing the server's
        response to the PUT request.
      On failure, raises a RequestError.
    """
    return self.Update(updated_profile, auth_token=auth_token, force=force, **kwargs)

  UpdateProfile = update_profile

  def execute_batch(self, batch_feed, url=DEFAULT_BATCH_URL, desired_class=None,
                    auth_token=None, **kwargs):
    """Sends a batch request feed to the server.

    Args:
      batch_feed: gdata.contacts.ContactFeed A feed containing batch
          request entries. Each entry contains the operation to be performed
          on the data contained in the entry. For example an entry with an
          operation type of insert will be used as if the individual entry
          had been inserted.
      url: str The batch URL to which these operations should be applied.
      converter: Function (optional) The function used to convert the server's
          response to an object.

    Returns:
      The results of the batch request's execution on the server. If the
      default converter is used, this is stored in a ContactsFeed.
    """
    return self.Post(batch_feed, url, desired_class=desired_class,
                     auth_token=None, **kwargs)

  ExecuteBatch = execute_batch

  def execute_batch_profiles(self, batch_feed, url=None,
                             desired_class=gdata.contacts.data.ProfilesFeed,
                             auth_token=None, **kwargs):
    """Sends a batch request feed to the server.

    Args:
      batch_feed: gdata.profiles.ProfilesFeed A feed containing batch
          request entries. Each entry contains the operation to be performed
          on the data contained in the entry. For example an entry with an
          operation type of insert will be used as if the individual entry
          had been inserted.
      url: string The batch URL to which these operations should be applied.
      converter: Function (optional) The function used to convert the server's
          response to an object. The default value is
          gdata.profiles.ProfilesFeedFromString.

    Returns:
      The results of the batch request's execution on the server. If the
      default converter is used, this is stored in a ProfilesFeed.
    """
    url = url or (DEFAULT_PROFILES_BATCH_URL % self.domain)
    return self.Post(batch_feed, url, desired_class=desired_class,
                     auth_token=auth_token, **kwargs)

  ExecuteBatchProfiles = execute_batch_profiles

  def _CleanUri(self, uri):
    """Sanitizes a feed URI.

    Args:
      uri: The URI to sanitize, can be relative or absolute.

    Returns:
      The given URI without its http://server prefix, if any.
      Keeps the leading slash of the URI.
    """
    url_prefix = 'http://%s' % self.server
    if uri.startswith(url_prefix):
      uri = uri[len(url_prefix):]
    return uri

class ContactsQuery(gdata.client.Query):
  """
  Create a custom Contacts Query

  Full specs can be found at: U{Contacts query parameters reference
  <http://code.google.com/apis/contacts/docs/3.0/reference.html#Parameters>}
  """

  def __init__(self, feed=None, group=None, orderby=None, showdeleted=None,
               sortorder=None, requirealldeleted=None, **kwargs):
    """
    @param max_results: The maximum number of entries to return. If you want
        to receive all of the contacts, rather than only the default maximum, you
        can specify a very large number for max-results.
    @param start-index: The 1-based index of the first result to be retrieved.
    @param updated-min: The lower bound on entry update dates.
    @param group: Constrains the results to only the contacts belonging to the
        group specified. Value of this parameter specifies group ID
    @param orderby:  Sorting criterion. The only supported value is
        lastmodified.
    @param showdeleted: Include deleted contacts in the returned contacts feed
    @pram sortorder: Sorting order direction. Can be either ascending or
        descending.
    @param requirealldeleted: Only relevant if showdeleted and updated-min
        are also provided. It dictates the behavior of the server in case it
        detects that placeholders of some entries deleted since the point in
        time specified as updated-min may have been lost.
    """
    gdata.client.Query.__init__(self, **kwargs)
    self.group = group
    self.orderby = orderby
    self.sortorder = sortorder
    self.showdeleted = showdeleted

  def modify_request(self, http_request):
    if self.group:
      gdata.client._add_query_param('group', self.group, http_request)
    if self.orderby:
      gdata.client._add_query_param('orderby', self.orderby, http_request)
    if self.sortorder:
      gdata.client._add_query_param('sortorder', self.sortorder, http_request)
    if self.showdeleted:
      gdata.client._add_query_param('showdeleted', self.showdeleted, http_request)
    gdata.client.Query.modify_request(self, http_request)

  ModifyRequest = modify_request


class ProfilesQuery(gdata.client.Query):
  """
  Create a custom Profiles Query

  Full specs can be found at: U{Profiless query parameters reference
  <http://code.google.com/apis/apps/profiles/reference.html#Parameters>}
  """

  def __init__(self, feed=None, start_key=None, **kwargs):
    """
    @param start_key: Opaque key of the first element to retrieve. Present in
        the next link of an earlier request, if further pages of response are
        available.
    """
    gdata.client.Query.__init__(self, **kwargs)
    self.feed = feed or 'https://www.google.com/m8/feeds/profiles/default/full'
    self.start_key = start_key

  def modify_request(self, http_request):
    if self.start_key:
      gdata.client._add_query_param('start-key', self.start_key, http_request)
    gdata.client.Query.modify_request(self, http_request)

  ModifyRequest = modify_request



"""PyCrypto RC4 implementation."""

from cryptomath import *
from RC4 import *

if pycryptoLoaded:
    import Crypto.Cipher.ARC4

    def new(key):
        return PyCrypto_RC4(key)

    class PyCrypto_RC4(RC4):

        def __init__(self, key):
            RC4.__init__(self, key, "pycrypto")
            self.context = Crypto.Cipher.ARC4.new(key)

        def encrypt(self, plaintext):
            return self.context.encrypt(plaintext)

        def decrypt(self, ciphertext):
            return self.context.decrypt(ciphertext)


"""Import this module for easy access to TLS Lite objects.

The TLS Lite API consists of classes, functions, and variables spread
throughout this package.  Instead of importing them individually with::

    from tlslite.TLSConnection import TLSConnection
    from tlslite.HandshakeSettings import HandshakeSettings
    from tlslite.errors import *
    .
    .

It's easier to do::

    from tlslite.api import *

This imports all the important objects (TLSConnection, Checker,
HandshakeSettings, etc.) into the global namespace.  In particular, it
imports::

    from constants import AlertLevel, AlertDescription, Fault
    from errors import *
    from Checker import Checker
    from HandshakeSettings import HandshakeSettings
    from Session import Session
    from SessionCache import SessionCache
    from SharedKeyDB import SharedKeyDB
    from TLSConnection import TLSConnection
    from VerifierDB import VerifierDB
    from X509 import X509
    from X509CertChain import X509CertChain

    from integration.HTTPTLSConnection import HTTPTLSConnection
    from integration.POP3_TLS import POP3_TLS
    from integration.IMAP4_TLS import IMAP4_TLS
    from integration.SMTP_TLS import SMTP_TLS
    from integration.XMLRPCTransport import XMLRPCTransport
    from integration.TLSSocketServerMixIn import TLSSocketServerMixIn
    from integration.TLSAsyncDispatcherMixIn import TLSAsyncDispatcherMixIn
    from integration.TLSTwistedProtocolWrapper import TLSTwistedProtocolWrapper
    from utils.cryptomath import cryptlibpyLoaded, m2cryptoLoaded,
                                 gmpyLoaded, pycryptoLoaded, prngName
    from utils.keyfactory import generateRSAKey, parsePEMKey, parseXMLKey,
                                 parseAsPublicKey, parsePrivateKey
"""

from constants import AlertLevel, AlertDescription, Fault
from errors import *
from Checker import Checker
from HandshakeSettings import HandshakeSettings
from Session import Session
from SessionCache import SessionCache
from SharedKeyDB import SharedKeyDB
from TLSConnection import TLSConnection
from VerifierDB import VerifierDB
from X509 import X509
from X509CertChain import X509CertChain

from integration.HTTPTLSConnection import HTTPTLSConnection
from integration.TLSSocketServerMixIn import TLSSocketServerMixIn
from integration.TLSAsyncDispatcherMixIn import TLSAsyncDispatcherMixIn
from integration.POP3_TLS import POP3_TLS
from integration.IMAP4_TLS import IMAP4_TLS
from integration.SMTP_TLS import SMTP_TLS
from integration.XMLRPCTransport import XMLRPCTransport
try:
    import twisted
    del(twisted)
    from integration.TLSTwistedProtocolWrapper import TLSTwistedProtocolWrapper
except ImportError:
    pass

from utils.cryptomath import cryptlibpyLoaded, m2cryptoLoaded, gmpyLoaded, \
                             pycryptoLoaded, prngName
from utils.keyfactory import generateRSAKey, parsePEMKey, parseXMLKey, \
                             parseAsPublicKey, parsePrivateKey



#!/usr/bin/python
#
# Copyright (C) 2010-2011 Google Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


"""Extend the gdata client for the Content API for Shopping."""


__author__ = 'afshar (Ali Afshar), dhermes (Daniel Hermes)'


import urllib

import atom.data
import gdata.client
from gdata.contentforshopping.data import ClientAccount
from gdata.contentforshopping.data import ClientAccountFeed
from gdata.contentforshopping.data import DatafeedEntry
from gdata.contentforshopping.data import DatafeedFeed
from gdata.contentforshopping.data import DataQualityEntry
from gdata.contentforshopping.data import DataQualityFeed
from gdata.contentforshopping.data import InventoryFeed
from gdata.contentforshopping.data import ProductEntry
from gdata.contentforshopping.data import ProductFeed
from gdata.contentforshopping.data import UsersEntry
from gdata.contentforshopping.data import UsersFeed


CFS_VERSION = 'v1'
CFS_HOST = 'content.googleapis.com'
CFS_URI = 'https://%s/content' % CFS_HOST
CFS_PROJECTION = 'schema'


class ContentForShoppingClient(gdata.client.GDClient):
  """Client for Content for Shopping API.

  :param account_id: Merchant account ID. This value will be used by default
                     for all requests, but may be overridden on a
                     request-by-request basis.
  :param api_version: The version of the API to target. Default value: 'v1'.
  :param **kwargs: Pass all addtional keywords to the GDClient constructor.
  """

  api_version = '1.0'

  def __init__(self, account_id=None, api_version=CFS_VERSION,
               cfs_uri=CFS_URI, **kwargs):
    self.cfs_account_id = account_id
    self.cfs_api_version = api_version
    self.cfs_uri = cfs_uri
    gdata.client.GDClient.__init__(self, **kwargs)

  def _create_uri(self, account_id, resource, path=(), use_projection=True,
                  dry_run=False, warnings=False, max_results=None,
                  start_token=None, start_index=None,
                  performance_start=None, performance_end=None):
    """Create a request uri from the given arguments.

    If arguments are None, use the default client attributes.
    """
    account_id = account_id or self.cfs_account_id
    if account_id is None:
        raise ValueError('No Account ID set. '
                         'Either set for the client, or per request')
    segments = [self.cfs_uri, self.cfs_api_version, account_id, resource]
    if use_projection:
      segments.append(CFS_PROJECTION)
    segments.extend(urllib.quote(value) for value in path)
    result = '/'.join(segments)

    request_params = []
    if dry_run:
      request_params.append('dry-run')
    if warnings:
      request_params.append('warnings')
    if max_results is not None:
      request_params.append('max-results=%s' % max_results)
    if start_token is not None:
      request_params.append('start-token=%s' % start_token)
    if start_index is not None:
      request_params.append('start-index=%s' % start_index)
    if performance_start is not None:
      request_params.append('performance.start=%s' % performance_start)
    if performance_end is not None:
      request_params.append('performance.end=%s' % performance_end)
    request_params = '&'.join(request_params)

    if request_params:
      result = '%s?%s' % (result, request_params)

    return result

  def _create_product_id(self, id, country, language, channel='online'):
    return '%s:%s:%s:%s' % (channel, language, country, id)

  def _create_batch_feed(self, entries, operation, feed=None,
                         feed_class=ProductFeed):
    if feed is None:
      feed = feed_class()
    for entry in entries:
      entry.batch_operation = gdata.data.BatchOperation(type=operation)
      feed.entry.append(entry)
    return feed

  # Operations on a single product

  def get_product(self, id, country, language, account_id=None,
                  auth_token=None):
    """Get a product by id, country and language.

    :param id: The product ID
    :param country: The country (target_country)
    :param language: The language (content_language)
    :param account_id: The Merchant Center Account ID. If ommitted the default
                       Account ID will be used for this client
    :param auth_token: An object which sets the Authorization HTTP header in its
                       modify_request method.
    """
    pid = self._create_product_id(id, country, language)
    uri = self._create_uri(account_id, 'items/products', path=[pid])
    return self.get_entry(uri, desired_class=ProductEntry,
                          auth_token=auth_token)

  GetProduct = get_product

  def insert_product(self, product, account_id=None, auth_token=None,
                     dry_run=False, warnings=False):
    """Create a new product, by posting the product entry feed.

    :param product: A :class:`gdata.contentforshopping.data.ProductEntry` with
                    the required product data.
    :param account_id: The Merchant Center Account ID. If ommitted the default
                       Account ID will be used for this client
    :param auth_token: An object which sets the Authorization HTTP header in its
                       modify_request method.
    :param dry_run: Flag to run all requests that modify persistent data in
                    dry-run mode. False by default.
    :param warnings: Flag to include warnings in response. False by default.
    """
    uri = self._create_uri(account_id, 'items/products',
                           dry_run=dry_run, warnings=warnings)
    return self.post(product, uri=uri, auth_token=auth_token)

  InsertProduct = insert_product

  def update_product(self, product, account_id=None, auth_token=None,
                     dry_run=False, warnings=False):
    """Update a product, by putting the product entry feed.

    :param product: A :class:`gdata.contentforshopping.data.ProductEntry` with
                    the required product data.
    :param account_id: The Merchant Center Account ID. If ommitted the default
                       Account ID will be used for this client
    :param auth_token: An object which sets the Authorization HTTP header in its
                       modify_request method.
    :param dry_run: Flag to run all requests that modify persistent data in
                    dry-run mode. False by default.
    :param warnings: Flag to include warnings in response. False
                     by default.
    """
    pid = self._create_product_id(product.product_id.text,
                                  product.target_country.text,
                                  product.content_language.text)
    uri = self._create_uri(account_id, 'items/products', path=[pid],
                           dry_run=dry_run, warnings=warnings)
    return self.update(product, uri=uri, auth_token=auth_token)

  UpdateProduct = update_product

  def delete_product(self, product, account_id=None, auth_token=None,
                     dry_run=False, warnings=False):
    """Delete a product

    :param product: A :class:`gdata.contentforshopping.data.ProductEntry` with
                    the required product data.
    :param account_id: The Merchant Center Account ID. If ommitted the default
                       Account ID will be used for this client
    :param auth_token: An object which sets the Authorization HTTP header in its
                       modify_request method.
    :param dry_run: Flag to run all requests that modify persistent data in
                    dry-run mode. False by default.
    :param warnings: Flag to include warnings in response. False by default.
    """
    pid = self._create_product_id(product.product_id.text,
                                  product.target_country.text,
                                  product.content_language.text)
    uri = self._create_uri(account_id, 'items/products', path=[pid],
                           dry_run=dry_run, warnings=warnings)
    return self.delete(uri, auth_token=auth_token)

  DeleteProduct = delete_product

  # Operations on multiple products

  def get_products(self, max_results=None, start_token=None, start_index=None,
                   performance_start=None, performance_end=None,
                   account_id=None, auth_token=None):
    """Get a feed of products for the account.

    :param max_results: The maximum number of results to return (default 25,
                        maximum 250).
    :param start_token: The start token of the feed provided by the API.
    :param start_index: The starting index of the feed to return (default 1,
                        maximum 10000)
    :param performance_start: The start date (inclusive) of click data returned.
                              Should be represented as YYYY-MM-DD; not appended
                              if left as None.
    :param performance_end: The end date (inclusive) of click data returned.
                            Should be represented as YYYY-MM-DD; not appended
                            if left as None.
    :param account_id: The Merchant Center Account ID. If ommitted the default
                       Account ID will be used for this client
    :param auth_token: An object which sets the Authorization HTTP header in its
                       modify_request method.
    """
    uri = self._create_uri(account_id, 'items/products',
                           max_results=max_results,
                           start_token=start_token,
                           start_index=start_index,
                           performance_start=performance_start,
                           performance_end=performance_end)
    return self.get_feed(uri, auth_token=auth_token,
        desired_class=ProductFeed)

  GetProducts = get_products

  def batch(self, feed, account_id=None, auth_token=None,
            dry_run=False, warnings=False):
    """Send a batch request.

    :param feed: The feed of batch entries to send.
    :param account_id: The Merchant Center Account ID. If ommitted the default
                       Account ID will be used for this client
    :param auth_token: An object which sets the Authorization HTTP header in its
                       modify_request method.
    :param dry_run: Flag to run all requests that modify persistent data in
                    dry-run mode. False by default.
    :param warnings: Flag to include warnings in response. False by default.
    """
    uri = self._create_uri(account_id, 'items/products', path=['batch'],
                           dry_run=dry_run, warnings=warnings)
    return self.post(feed, uri=uri, auth_token=auth_token,
                     desired_class=ProductFeed)

  Batch = batch

  def insert_products(self, products, account_id=None, auth_token=None,
                      dry_run=False, warnings=False):
    """Insert the products using a batch request

    :param products: A list of product entries
    :param account_id: The Merchant Center Account ID. If ommitted the default
                       Account ID will be used for this client
    :param auth_token: An object which sets the Authorization HTTP header in its
                       modify_request method.
    :param dry_run: Flag to run all requests that modify persistent data in
                    dry-run mode. False by default.
    :param warnings: Flag to include warnings in response. False by default.
    """
    feed = self._create_batch_feed(products, 'insert')
    return self.batch(feed, account_id=account_id, auth_token=auth_token,
                      dry_run=dry_run, warnings=warnings)

  InsertProducts = insert_products

  def update_products(self, products, account_id=None, auth_token=None,
                      dry_run=False, warnings=False):
    """Update the products using a batch request

    :param products: A list of product entries
    :param account_id: The Merchant Center Account ID. If ommitted the default
                       Account ID will be used for this client
    :param auth_token: An object which sets the Authorization HTTP header in its
                       modify_request method.
    :param dry_run: Flag to run all requests that modify persistent data in
                    dry-run mode. False by default.
    :param warnings: Flag to include warnings in response. False by default.

    .. note:: Entries must have the atom:id element set.
    """
    feed = self._create_batch_feed(products, 'update')
    return self.batch(feed, account_id=account_id, auth_token=auth_token,
                      dry_run=dry_run, warnings=warnings)

  UpdateProducts = update_products

  def delete_products(self, products, account_id=None, auth_token=None,
                      dry_run=False, warnings=False):
    """Delete the products using a batch request.

    :param products: A list of product entries
    :param account_id: The Merchant Center Account ID. If ommitted the default
                       Account ID will be used for this client
    :param auth_token: An object which sets the Authorization HTTP header in its
                       modify_request method.
    :param dry_run: Flag to run all requests that modify persistent data in
                    dry-run mode. False by default.
    :param warnings: Flag to include warnings in response. False by default.

    .. note:: Entries must have the atom:id element set.
    """
    feed = self._create_batch_feed(products, 'delete')
    return self.batch(feed, account_id=account_id, auth_token=auth_token,
                      dry_run=dry_run, warnings=warnings)

  DeleteProducts = delete_products

  # Operations on datafeeds

  def get_datafeeds(self, account_id=None):
    """Get the feed of datafeeds.

    :param account_id: The Sub-Account ID. If ommitted the default
                       Account ID will be used for this client.
    """
    uri = self._create_uri(account_id, 'datafeeds/products',
                           use_projection=False)
    return self.get_feed(uri, desired_class=DatafeedFeed)

  GetDatafeeds = get_datafeeds

  # Operations on a single datafeed

  def get_datafeed(self, feed_id, account_id=None, auth_token=None):
    """Get the feed of a single datafeed.

    :param feed_id: The ID of the desired datafeed.
    :param account_id: The Sub-Account ID. If ommitted the default
                       Account ID will be used for this client.
    :param auth_token: An object which sets the Authorization HTTP header in its
                       modify_request method.
    """
    uri = self._create_uri(account_id, 'datafeeds/products', path=[feed_id],
                           use_projection=False)
    return self.get_feed(uri, auth_token=auth_token,
                         desired_class=DatafeedEntry)

  GetDatafeed = get_datafeed

  def insert_datafeed(self, entry, account_id=None, auth_token=None,
                      dry_run=False, warnings=False):
    """Insert a datafeed.

    :param entry: XML Content of post request required for registering a
                  datafeed.
    :param account_id: The Sub-Account ID. If ommitted the default
                       Account ID will be used for this client.
    :param auth_token: An object which sets the Authorization HTTP header in its
                       modify_request method.
    :param dry_run: Flag to run all requests that modify persistent data in
                    dry-run mode. False by default.
    :param warnings: Flag to include warnings in response. False by default.
    """
    uri = self._create_uri(account_id, 'datafeeds/products',
                           use_projection=False, dry_run=dry_run,
                           warnings=warnings)
    return self.post(entry, uri=uri, auth_token=auth_token)

  InsertDatafeed = insert_datafeed

  def update_datafeed(self, entry, feed_id, account_id=None, auth_token=None,
                      dry_run=False, warnings=False):
    """Update the feed of a single datafeed.

    :param entry: XML Content of put request required for updating a
                  datafeed.
    :param feed_id: The ID of the desired datafeed.
    :param account_id: The Sub-Account ID. If ommitted the default
                       Account ID will be used for this client.
    :param auth_token: An object which sets the Authorization HTTP header in its
                       modify_request method.
    :param dry_run: Flag to run all requests that modify persistent data in
                    dry-run mode. False by default.
    :param warnings: Flag to include warnings in response. False by default.
    """
    uri = self._create_uri(account_id, 'datafeeds/products', path=[feed_id],
                           use_projection=False, dry_run=dry_run,
                           warnings=warnings)
    return self.update(entry, auth_token=auth_token, uri=uri)

  UpdateDatafeed = update_datafeed

  def delete_datafeed(self, feed_id, account_id=None, auth_token=None):
    """Delete a single datafeed.

    :param feed_id: The ID of the desired datafeed.
    :param account_id: The Sub-Account ID. If ommitted the default
                       Account ID will be used for this client.
    :param auth_token: An object which sets the Authorization HTTP header in its
                       modify_request method.
    """
    uri = self._create_uri(account_id, 'datafeeds/products', path=[feed_id],
                           use_projection=False)
    return self.delete(uri, auth_token=auth_token)

  DeleteDatafeed = delete_datafeed

  # Operations on client accounts

  def get_client_accounts(self, max_results=None, start_index=None,
                          account_id=None, auth_token=None):
    """Get the feed of managed accounts

    :param max_results: The maximum number of results to return (default 25,
                        maximum 250).
    :param start_index: The starting index of the feed to return (default 1,
                        maximum 10000)
    :param account_id: The Merchant Center Account ID. If ommitted the default
                       Account ID will be used for this client
    :param auth_token: An object which sets the Authorization HTTP header in its
                       modify_request method.
    """
    uri = self._create_uri(account_id, 'managedaccounts',
                           max_results=max_results, start_index=start_index,
                           use_projection=False)
    return self.get_feed(uri, desired_class=ClientAccountFeed,
                         auth_token=auth_token)

  GetClientAccounts = get_client_accounts

  def get_client_account(self, client_account_id,
                         account_id=None, auth_token=None):
    """Get a managed account.

    :param client_account_id: The Account ID of the subaccount being retrieved.
    :param account_id: The Merchant Center Account ID. If ommitted the default
                       Account ID will be used for this client
    :param auth_token: An object which sets the Authorization HTTP header in its
                       modify_request method.
    """
    uri = self._create_uri(account_id, 'managedaccounts',
                           path=[client_account_id], use_projection=False)
    return self.get_entry(uri, desired_class=ClientAccount,
                          auth_token=auth_token)

  GetClientAccount = get_client_account

  def insert_client_account(self, entry, account_id=None, auth_token=None,
                            dry_run=False, warnings=False):
    """Insert a client account entry

    :param entry: An entry of type ClientAccount
    :param account_id: The Merchant Center Account ID. If ommitted the default
                       Account ID will be used for this client
    :param auth_token: An object which sets the Authorization HTTP header in its
                       modify_request method.
    :param dry_run: Flag to run all requests that modify persistent data in
                    dry-run mode. False by default.
    :param warnings: Flag to include warnings in response. False by default.
    """
    uri = self._create_uri(account_id, 'managedaccounts',
                           use_projection=False, dry_run=dry_run,
                           warnings=warnings)
    return self.post(entry, uri=uri, auth_token=auth_token)

  InsertClientAccount = insert_client_account

  def update_client_account(self, entry, client_account_id, account_id=None,
                            auth_token=None, dry_run=False, warnings=False):
    """Update a client account

    :param entry: An entry of type ClientAccount to update to
    :param client_account_id: The client account ID
    :param account_id: The Merchant Center Account ID. If ommitted the default
                       Account ID will be used for this client
    :param auth_token: An object which sets the Authorization HTTP header in its
                       modify_request method.
    :param dry_run: Flag to run all requests that modify persistent data in
                    dry-run mode. False by default.
    :param warnings: Flag to include warnings in response. False by default.
    """
    uri = self._create_uri(account_id, 'managedaccounts',
                           path=[client_account_id], use_projection=False,
                           dry_run=dry_run, warnings=warnings)
    return self.update(entry, uri=uri, auth_token=auth_token)

  UpdateClientAccount = update_client_account

  def delete_client_account(self, client_account_id, account_id=None,
                            auth_token=None, dry_run=False, warnings=False):
    """Delete a client account

    :param client_account_id: The client account ID
    :param account_id: The Merchant Center Account ID. If ommitted the default
                       Account ID will be used for this client
    :param auth_token: An object which sets the Authorization HTTP header in its
                       modify_request method.
    :param dry_run: Flag to run all requests that modify persistent data in
                    dry-run mode. False by default.
    :param warnings: Flag to include warnings in response. False by default.
    """

    uri = self._create_uri(account_id, 'managedaccounts',
                           path=[client_account_id], use_projection=False,
                           dry_run=dry_run, warnings=warnings)
    return self.delete(uri, auth_token=auth_token)

  DeleteClientAccount = delete_client_account

  def get_users_feed(self, account_id=None, auth_token=None):
    """Get the users feed for an account.

    :param account_id: The Merchant Center Account ID. If ommitted the default
                       Account ID will be used for this client
    :param auth_token: An object which sets the Authorization HTTP header in its
                       modify_request method.
    """

    uri = self._create_uri(account_id, 'users', use_projection=False)
    return self.get_feed(uri, auth_token=auth_token, desired_class=UsersFeed)

  GetUsersFeed = get_users_feed

  def get_users_entry(self, user_email, account_id=None, auth_token=None):
    """Get a users feed entry for an account.

    :param user_email: Email of the user entry to be retrieved.
    :param account_id: The Merchant Center Account ID. If ommitted the default
                       Account ID will be used for this client
    :param auth_token: An object which sets the Authorization HTTP header in its
                       modify_request method.
    """
    uri = self._create_uri(
        account_id, 'users', path=[user_email], use_projection=False)
    return self.get_entry(uri, auth_token=auth_token, desired_class=UsersEntry)

  GetUsersEntry = get_users_entry

  def insert_users_entry(self, entry, account_id=None, auth_token=None):
    """Insert a users feed entry for an account.

    :param entry: A :class:`gdata.contentforshopping.data.UsersEntry` with
                  the required user data.
    :param account_id: The Merchant Center Account ID. If ommitted the default
                       Account ID will be used for this client
    :param auth_token: An object which sets the Authorization HTTP header in its
                       modify_request method.
    """
    uri = self._create_uri(account_id, 'users', use_projection=False)
    return self.post(entry, uri=uri, auth_token=auth_token)

  InsertUsersEntry = insert_users_entry

  def update_users_entry(self, entry, account_id=None, auth_token=None):
    """Update a users feed entry for an account.

    :param entry: A :class:`gdata.contentforshopping.data.UsersEntry` with
                  the required user data.
    :param account_id: The Merchant Center Account ID. If ommitted the default
                       Account ID will be used for this client
    :param auth_token: An object which sets the Authorization HTTP header in its
                       modify_request method.
    """
    # Could also use entry.find_edit_link() but that is inconsistent
    # with the rest of the module
    user_email = entry.title.text
    uri = self._create_uri(
        account_id, 'users', path=[user_email], use_projection=False)
    return self.update(entry, uri=uri, auth_token=auth_token)

  UpdateUsersEntry = update_users_entry

  def delete_users_entry(self, entry, account_id=None, auth_token=None):
    """Delete a users feed entry for an account.

    :param entry: A :class:`gdata.contentforshopping.data.UsersEntry` with
                  the required user data.
    :param account_id: The Merchant Center Account ID. If ommitted the default
                       Account ID will be used for this client
    :param auth_token: An object which sets the Authorization HTTP header in its
                       modify_request method.
    """
    # Could also use entry.find_edit_link() but that is inconsistent
    # with the rest of the module
    user_email = entry.title.text
    uri = self._create_uri(
        account_id, 'users', path=[user_email], use_projection=False)
    return self.delete(uri, auth_token=auth_token)

  DeleteUsersEntry = delete_users_entry

  def get_data_quality_feed(self, account_id=None, auth_token=None,
                            max_results=None, start_index=None):
    """Get the data quality feed for an account.

    :param max_results: The maximum number of results to return (default 25,
                        max 100).
    :param start_index: The starting index of the feed to return.
    :param account_id: The Merchant Center Account ID. If ommitted the default
                       Account ID will be used for this client
    :param auth_token: An object which sets the Authorization HTTP header in its
                       modify_request method.
    """

    uri = self._create_uri(account_id, 'dataquality', use_projection=False,
                           max_results=max_results, start_index=start_index)
    return self.get_feed(uri, auth_token=auth_token,
                         desired_class=DataQualityFeed)

  GetDataQualityFeed = get_data_quality_feed

  def get_data_quality_entry(self, secondary_account_id=None,
                             account_id=None, auth_token=None):
    """Get the data quality feed entry for an account.

    :param secondary_account_id: The Account ID of the secondary account. If
                                 ommitted the value of account_id is used.
    :param account_id: The Merchant Center Account ID. If ommitted the default
                       Account ID will be used for this client
    :param auth_token: An object which sets the Authorization HTTP header in its
                       modify_request method.
    """
    if secondary_account_id is None:
      secondary_account_id = account_id or self.cfs_account_id

    uri = self._create_uri(account_id, 'dataquality',
                           path=[secondary_account_id],
                           use_projection=False)
    return self.get_entry(uri, auth_token=auth_token,
                          desired_class=DataQualityEntry)

  GetDataQualityEntry = get_data_quality_entry

  def update_inventory_entry(self, product, id, country, language, store_code,
                             account_id=None, auth_token=None):
    """Make a local product update, by putting the inventory entry.

    :param product: A :class:`gdata.contentforshopping.data.InventoryEntry`
                    with the required product data.
    :param id: The product ID
    :param country: The country (target_country)
    :param language: The language (content_language)
    :param store_code: The code for the store where this local product will
                       be updated.
    :param account_id: The Merchant Center Account ID. If ommitted the default
                       Account ID will be used for this client
    :param auth_token: An object which sets the Authorization HTTP header in its
                       modify_request method.
    """
    pid = self._create_product_id(id, country, language, channel='local')
    uri = self._create_uri(account_id, 'inventory',
                           path=[store_code, 'items', pid],
                           use_projection=False)
    return self.update(product, uri=uri, auth_token=auth_token)

  UpdateInventoryEntry = update_inventory_entry

  def add_local_id(self, product, id, country, language,
                   store_code, account_id=None):
    """Add an atom id to a local product with a local store specific URI.

    :param product: A :class:`gdata.contentforshopping.data.InventoryEntry`
                    with the required product data.
    :param id: The product ID
    :param country: The country (target_country)
    :param language: The language (content_language)
    :param store_code: The code for the store where this local product will
                       be updated.
    :param account_id: The Merchant Center Account ID. If ommitted the default
                       Account ID will be used for this client
    """
    pid = self._create_product_id(id, country, language, channel='local')
    uri = self._create_uri(account_id, 'inventory',
                           path=[store_code, 'items', pid],
                           use_projection=False)
    product.id = atom.data.Id(uri)
    return product

  AddLocalId = add_local_id

  def update_inventory_feed(self, products, account_id=None, auth_token=None):
    """Update a batch of local products, by putting the product entry feed.

    :param products: A list containing entries of
                     :class:`gdata.contentforshopping.data.InventoryEntry`
                     with the required product data
    :param account_id: The Merchant Center Account ID. If ommitted the default
                       Account ID will be used for this client
    :param auth_token: An object which sets the Authorization HTTP header in its
                       modify_request method.

    .. note:: Entries must have the atom:id element set. You can use
              add_local_id to set this attribute using the store_code, product
              id, country and language.
    """
    feed = self._create_batch_feed(products, 'update',
                                   feed_class=InventoryFeed)
    uri = self._create_uri(account_id, 'inventory', path=['batch'],
                           use_projection=False)
    return self.post(feed, uri=uri, auth_token=auth_token)

  UpdateInventoryFeed = update_inventory_feed



#!/usr/bin/python2.4
#
# Copyright 2011 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Data model classes for the Organization Unit Provisioning API."""


__author__ = 'Gunjan Sharma <gunjansharma@google.com>'


import gdata.apps
import gdata.apps.apps_property_entry
import gdata.apps_property
import gdata.data


# This is required to work around a naming conflict between the Google
# Spreadsheets API and Python's built-in property function
pyproperty = property


# The apps:property name of an organization unit
ORG_UNIT_NAME = 'name'
# The apps:property orgUnitPath of an organization unit
ORG_UNIT_PATH = 'orgUnitPath'
# The apps:property parentOrgUnitPath of an organization unit
PARENT_ORG_UNIT_PATH = 'parentOrgUnitPath'
# The apps:property description of an organization unit
ORG_UNIT_DESCRIPTION = 'description'
# The apps:property blockInheritance of an organization unit
ORG_UNIT_BLOCK_INHERITANCE = 'blockInheritance'
# The apps:property userEmail of a user entry
USER_EMAIL = 'orgUserEmail'
# The apps:property list of users to move
USERS_TO_MOVE = 'usersToMove'
# The apps:property list of moved users
MOVED_USERS = 'usersMoved'
# The apps:property customerId for the domain
CUSTOMER_ID = 'customerId'
# The apps:property name of the customer org unit
CUSTOMER_ORG_UNIT_NAME = 'customerOrgUnitName'
# The apps:property description of the customer org unit
CUSTOMER_ORG_UNIT_DESCRIPTION = 'customerOrgUnitDescription'
# The apps:property old organization unit's path for a user
OLD_ORG_UNIT_PATH = 'oldOrgUnitPath'


class CustomerIdEntry(gdata.apps.apps_property_entry.AppsPropertyEntry):
  """Represents a customerId entry in object form."""

  def GetCustomerId(self):
    """Get the customer ID of the customerId object.

    Returns:
      The customer ID of this customerId object as a string or None.
    """
    return self._GetProperty(CUSTOMER_ID)

  customer_id = pyproperty(GetCustomerId)

  def GetOrgUnitName(self):
    """Get the Organization Unit name of the customerId object.

    Returns:
      The Organization unit name of this customerId object as a string or None.
    """
    return self._GetProperty(ORG_UNIT_NAME)

  org_unit_name = pyproperty(GetOrgUnitName)

  def GetCustomerOrgUnitName(self):
    """Get the Customer Organization Unit name of the customerId object.

    Returns:
      The Customer Organization unit name of this customerId object as a string
      or None.
    """
    return self._GetProperty(CUSTOMER_ORG_UNIT_NAME)

  customer_org_unit_name = pyproperty(GetCustomerOrgUnitName)

  def GetOrgUnitDescription(self):
    """Get the Organization Unit Description of the customerId object.

    Returns:
      The Organization Unit Description of this customerId object as a string
          or None.
    """
    return self._GetProperty(ORG_UNIT_DESCRIPTION)

  org_unit_description = pyproperty(GetOrgUnitDescription)

  def GetCustomerOrgUnitDescription(self):
    """Get the Customer Organization Unit Description of the customerId object.

    Returns:
      The Customer Organization Unit Description of this customerId object
          as a string or None.
    """
    return self._GetProperty(CUSTOMER_ORG_UNIT_DESCRIPTION)

  customer_org_unit_description = pyproperty(GetCustomerOrgUnitDescription)


class OrgUnitEntry(gdata.apps.apps_property_entry.AppsPropertyEntry):
  """Represents an OrganizationUnit in object form."""

  def GetOrgUnitName(self):
    """Get the Organization Unit name of the OrganizationUnit object.

    Returns:
      The Organization unit name of this OrganizationUnit object as a string
      or None.
    """
    return self._GetProperty(ORG_UNIT_NAME)

  def SetOrgUnitName(self, value):
    """Set the Organization Unit name of the OrganizationUnit object.

    Args:
      value: [string] The new Organization Unit name to give this object.
    """
    self._SetProperty(ORG_UNIT_NAME, value)

  org_unit_name = pyproperty(GetOrgUnitName, SetOrgUnitName)

  def GetOrgUnitPath(self):
    """Get the Organization Unit Path of the OrganizationUnit object.

    Returns:
      The Organization Unit Path of this OrganizationUnit object as a string
          or None.
    """
    return self._GetProperty(ORG_UNIT_PATH)

  def SetOrgUnitPath(self, value):
    """Set the Organization Unit path of the OrganizationUnit object.

    Args:
      value: [string] The new Organization Unit path to give this object.
    """
    self._SetProperty(ORG_UNIT_PATH, value)

  org_unit_path = pyproperty(GetOrgUnitPath, SetOrgUnitPath)

  def GetParentOrgUnitPath(self):
    """Get the Parent Organization Unit Path of the OrganizationUnit object.

    Returns:
      The Parent Organization Unit Path of this OrganizationUnit object
      as a string or None.
    """
    return self._GetProperty(PARENT_ORG_UNIT_PATH)

  def SetParentOrgUnitPath(self, value):
    """Set the Parent Organization Unit path of the OrganizationUnit object.

    Args:
      value: [string] The new Parent Organization Unit path
             to give this object.
    """
    self._SetProperty(PARENT_ORG_UNIT_PATH, value)

  parent_org_unit_path = pyproperty(GetParentOrgUnitPath, SetParentOrgUnitPath)

  def GetOrgUnitDescription(self):
    """Get the Organization Unit Description of the OrganizationUnit object.

    Returns:
      The Organization Unit Description of this OrganizationUnit object
      as a string or None.
    """
    return self._GetProperty(ORG_UNIT_DESCRIPTION)

  def SetOrgUnitDescription(self, value):
    """Set the Organization Unit Description of the OrganizationUnit object.

    Args:
      value: [string] The new Organization Unit Description
             to give this object.
    """
    self._SetProperty(ORG_UNIT_DESCRIPTION, value)

  org_unit_description = pyproperty(GetOrgUnitDescription,
                                    SetOrgUnitDescription)

  def GetOrgUnitBlockInheritance(self):
    """Get the block_inheritance flag of the OrganizationUnit object.

    Returns:
      The the block_inheritance flag of this OrganizationUnit object
      as a string or None.
    """
    return self._GetProperty(ORG_UNIT_BLOCK_INHERITANCE)

  def SetOrgUnitBlockInheritance(self, value):
    """Set the block_inheritance flag of the OrganizationUnit object.

    Args:
      value: [string] The new block_inheritance flag to give this object.
    """
    self._SetProperty(ORG_UNIT_BLOCK_INHERITANCE, value)

  org_unit_block_inheritance = pyproperty(GetOrgUnitBlockInheritance,
                                          SetOrgUnitBlockInheritance)

  def GetMovedUsers(self):
    """Get the moved users of the OrganizationUnit object.

    Returns:
      The the moved users of this OrganizationUnit object as a string or None.
    """
    return self._GetProperty(MOVED_USERS)

  def SetUsersToMove(self, value):
    """Set the Users to Move of the OrganizationUnit object.

    Args:
      value: [string] The comma seperated list of users to move
             to give this object.
    """
    self._SetProperty(USERS_TO_MOVE, value)

  move_users = pyproperty(GetMovedUsers, SetUsersToMove)

  def __init__(
      self, org_unit_name=None, org_unit_path=None,
      parent_org_unit_path=None, org_unit_description=None,
      org_unit_block_inheritance=None, move_users=None, *args, **kwargs):
    """Constructs a new OrganizationUnit object with the given arguments.

    Args:
      org_unit_name: string (optional) The organization unit name
          for the object.
      org_unit_path: string (optional) The organization unit path
          for the object.
      parent_org_unit_path: string (optional) The parent organization unit path
          for the object.
      org_unit_description: string (optional) The organization unit description
          for the object.
      org_unit_block_inheritance: boolean (optional) weather or not inheritance
          from the organization unit is blocked.
      move_users: string (optional) comma seperated list of users to move.
      args: The other parameters to pass to gdata.entry.GDEntry constructor.
      kwargs: The other parameters to pass to gdata.entry.GDEntry constructor.
    """
    super(OrgUnitEntry, self).__init__(*args, **kwargs)
    if org_unit_name:
      self.org_unit_name = org_unit_name
    if org_unit_path:
      self.org_unit_path = org_unit_path
    if parent_org_unit_path:
      self.parent_org_unit_path = parent_org_unit_path
    if org_unit_description:
      self.org_unit_description = org_unit_description
    if org_unit_block_inheritance is not None:
      self.org_unit_block_inheritance = str(org_unit_block_inheritance)
    if move_users:
      self.move_users = move_users


class OrgUnitFeed(gdata.data.GDFeed):
  """Represents a feed of OrgUnitEntry objects."""

  # Override entry so that this feed knows how to type its list of entries.
  entry = [OrgUnitEntry]


class OrgUserEntry(gdata.apps.apps_property_entry.AppsPropertyEntry):
  """Represents an OrgUser in object form."""

  def GetUserEmail(self):
    """Get the user email address of the OrgUser object.

    Returns:
      The user email address of this OrgUser object as a string or None.
    """
    return self._GetProperty(USER_EMAIL)

  def SetUserEmail(self, value):
    """Set the user email address of this OrgUser object.

    Args:
      value: string The new user email address to give this object.
    """
    self._SetProperty(USER_EMAIL, value)

  user_email = pyproperty(GetUserEmail, SetUserEmail)

  def GetOrgUnitPath(self):
    """Get the Organization Unit Path of the OrgUser object.

    Returns:
      The Organization Unit Path of this OrgUser object as a string or None.
    """
    return self._GetProperty(ORG_UNIT_PATH)

  def SetOrgUnitPath(self, value):
    """Set the Organization Unit path of the OrgUser object.

    Args:
      value: [string] The new Organization Unit path to give this object.
    """
    self._SetProperty(ORG_UNIT_PATH, value)

  org_unit_path = pyproperty(GetOrgUnitPath, SetOrgUnitPath)

  def GetOldOrgUnitPath(self):
    """Get the Old Organization Unit Path of the OrgUser object.

    Returns:
      The Old Organization Unit Path of this OrgUser object as a string
      or None.
    """
    return self._GetProperty(OLD_ORG_UNIT_PATH)

  def SetOldOrgUnitPath(self, value):
    """Set the Old Organization Unit path of the OrgUser object.

    Args:
      value: [string] The new Old Organization Unit path to give this object.
    """
    self._SetProperty(OLD_ORG_UNIT_PATH, value)

  old_org_unit_path = pyproperty(GetOldOrgUnitPath, SetOldOrgUnitPath)

  def __init__(
      self, user_email=None, org_unit_path=None,
      old_org_unit_path=None, *args, **kwargs):
    """Constructs a new OrgUser object with the given arguments.

    Args:
      user_email: string (optional) The user email address for the object.
      org_unit_path: string (optional) The organization unit path
          for the object.
      old_org_unit_path: string (optional) The old organization unit path
          for the object.
      args: The other parameters to pass to gdata.entry.GDEntry constructor.
      kwargs: The other parameters to pass to gdata.entry.GDEntry constructor.
    """
    super(OrgUserEntry, self).__init__(*args, **kwargs)
    if user_email:
      self.user_email = user_email
    if org_unit_path:
      self.org_unit_path = org_unit_path
    if old_org_unit_path:
      self.old_org_unit_path = old_org_unit_path


class OrgUserFeed(gdata.data.GDFeed):
  """Represents a feed of OrgUserEntry objects."""

  # Override entry so that this feed knows how to type its list of entries.
  entry = [OrgUserEntry]



"""cryptomath module

This module has basic math/crypto code."""

import os
import sys
import math
import base64
import binascii
if sys.version_info[:2] <= (2, 4):
  from sha import sha as sha1
else:
  from hashlib import sha1

from compat import *


# **************************************************************************
# Load Optional Modules
# **************************************************************************

# Try to load M2Crypto/OpenSSL
try:
    from M2Crypto import m2
    m2cryptoLoaded = True

except ImportError:
    m2cryptoLoaded = False


# Try to load cryptlib
try:
    import cryptlib_py
    try:
        cryptlib_py.cryptInit()
    except cryptlib_py.CryptException, e:
        #If tlslite and cryptoIDlib are both present,
        #they might each try to re-initialize this,
        #so we're tolerant of that.
        if e[0] != cryptlib_py.CRYPT_ERROR_INITED:
            raise
    cryptlibpyLoaded = True

except ImportError:
    cryptlibpyLoaded = False

#Try to load GMPY
try:
    import gmpy
    gmpyLoaded = True
except ImportError:
    gmpyLoaded = False

#Try to load pycrypto
try:
    import Crypto.Cipher.AES
    pycryptoLoaded = True
except ImportError:
    pycryptoLoaded = False


# **************************************************************************
# PRNG Functions
# **************************************************************************

# Get os.urandom PRNG
try:
    os.urandom(1)
    def getRandomBytes(howMany):
        return stringToBytes(os.urandom(howMany))
    prngName = "os.urandom"

except:
    # Else get cryptlib PRNG
    if cryptlibpyLoaded:
        def getRandomBytes(howMany):
            randomKey = cryptlib_py.cryptCreateContext(cryptlib_py.CRYPT_UNUSED,
                                                       cryptlib_py.CRYPT_ALGO_AES)
            cryptlib_py.cryptSetAttribute(randomKey,
                                          cryptlib_py.CRYPT_CTXINFO_MODE,
                                          cryptlib_py.CRYPT_MODE_OFB)
            cryptlib_py.cryptGenerateKey(randomKey)
            bytes = createByteArrayZeros(howMany)
            cryptlib_py.cryptEncrypt(randomKey, bytes)
            return bytes
        prngName = "cryptlib"

    else:
        #Else get UNIX /dev/urandom PRNG
        try:
            devRandomFile = open("/dev/urandom", "rb")
            def getRandomBytes(howMany):
                return stringToBytes(devRandomFile.read(howMany))
            prngName = "/dev/urandom"
        except IOError:
            #Else get Win32 CryptoAPI PRNG
            try:
                import win32prng
                def getRandomBytes(howMany):
                    s = win32prng.getRandomBytes(howMany)
                    if len(s) != howMany:
                        raise AssertionError()
                    return stringToBytes(s)
                prngName ="CryptoAPI"
            except ImportError:
                #Else no PRNG :-(
                def getRandomBytes(howMany):
                    raise NotImplementedError("No Random Number Generator "\
                                              "available.")
            prngName = "None"

# **************************************************************************
# Converter Functions
# **************************************************************************

def bytesToNumber(bytes):
    total = 0L
    multiplier = 1L
    for count in range(len(bytes)-1, -1, -1):
        byte = bytes[count]
        total += multiplier * byte
        multiplier *= 256
    return total

def numberToBytes(n):
    howManyBytes = numBytes(n)
    bytes = createByteArrayZeros(howManyBytes)
    for count in range(howManyBytes-1, -1, -1):
        bytes[count] = int(n % 256)
        n >>= 8
    return bytes

def bytesToBase64(bytes):
    s = bytesToString(bytes)
    return stringToBase64(s)

def base64ToBytes(s):
    s = base64ToString(s)
    return stringToBytes(s)

def numberToBase64(n):
    bytes = numberToBytes(n)
    return bytesToBase64(bytes)

def base64ToNumber(s):
    bytes = base64ToBytes(s)
    return bytesToNumber(bytes)

def stringToNumber(s):
    bytes = stringToBytes(s)
    return bytesToNumber(bytes)

def numberToString(s):
    bytes = numberToBytes(s)
    return bytesToString(bytes)

def base64ToString(s):
    try:
        return base64.decodestring(s)
    except binascii.Error, e:
        raise SyntaxError(e)
    except binascii.Incomplete, e:
        raise SyntaxError(e)

def stringToBase64(s):
    return base64.encodestring(s).replace("\n", "")

def mpiToNumber(mpi): #mpi is an openssl-format bignum string
    if (ord(mpi[4]) & 0x80) !=0: #Make sure this is a positive number
        raise AssertionError()
    bytes = stringToBytes(mpi[4:])
    return bytesToNumber(bytes)

def numberToMPI(n):
    bytes = numberToBytes(n)
    ext = 0
    #If the high-order bit is going to be set,
    #add an extra byte of zeros
    if (numBits(n) & 0x7)==0:
        ext = 1
    length = numBytes(n) + ext
    bytes = concatArrays(createByteArrayZeros(4+ext), bytes)
    bytes[0] = (length >> 24) & 0xFF
    bytes[1] = (length >> 16) & 0xFF
    bytes[2] = (length >> 8) & 0xFF
    bytes[3] = length & 0xFF
    return bytesToString(bytes)



# **************************************************************************
# Misc. Utility Functions
# **************************************************************************

def numBytes(n):
    if n==0:
        return 0
    bits = numBits(n)
    return int(math.ceil(bits / 8.0))

def hashAndBase64(s):
    return stringToBase64(sha1(s).digest())

def getBase64Nonce(numChars=22): #defaults to an 132 bit nonce
    bytes = getRandomBytes(numChars)
    bytesStr = "".join([chr(b) for b in bytes])
    return stringToBase64(bytesStr)[:numChars]


# **************************************************************************
# Big Number Math
# **************************************************************************

def getRandomNumber(low, high):
    if low >= high:
        raise AssertionError()
    howManyBits = numBits(high)
    howManyBytes = numBytes(high)
    lastBits = howManyBits % 8
    while 1:
        bytes = getRandomBytes(howManyBytes)
        if lastBits:
            bytes[0] = bytes[0] % (1 << lastBits)
        n = bytesToNumber(bytes)
        if n >= low and n < high:
            return n

def gcd(a,b):
    a, b = max(a,b), min(a,b)
    while b:
        a, b = b, a % b
    return a

def lcm(a, b):
    #This will break when python division changes, but we can't use // cause
    #of Jython
    return (a * b) / gcd(a, b)

#Returns inverse of a mod b, zero if none
#Uses Extended Euclidean Algorithm
def invMod(a, b):
    c, d = a, b
    uc, ud = 1, 0
    while c != 0:
        #This will break when python division changes, but we can't use //
        #cause of Jython
        q = d / c
        c, d = d-(q*c), c
        uc, ud = ud - (q * uc), uc
    if d == 1:
        return ud % b
    return 0


if gmpyLoaded:
    def powMod(base, power, modulus):
        base = gmpy.mpz(base)
        power = gmpy.mpz(power)
        modulus = gmpy.mpz(modulus)
        result = pow(base, power, modulus)
        return long(result)

else:
    #Copied from Bryan G. Olson's post to comp.lang.python
    #Does left-to-right instead of pow()'s right-to-left,
    #thus about 30% faster than the python built-in with small bases
    def powMod(base, power, modulus):
        nBitScan = 5

        """ Return base**power mod modulus, using multi bit scanning
        with nBitScan bits at a time."""

        #TREV - Added support for negative exponents
        negativeResult = False
        if (power < 0):
            power *= -1
            negativeResult = True

        exp2 = 2**nBitScan
        mask = exp2 - 1

        # Break power into a list of digits of nBitScan bits.
        # The list is recursive so easy to read in reverse direction.
        nibbles = None
        while power:
            nibbles = int(power & mask), nibbles
            power = power >> nBitScan

        # Make a table of powers of base up to 2**nBitScan - 1
        lowPowers = [1]
        for i in xrange(1, exp2):
            lowPowers.append((lowPowers[i-1] * base) % modulus)

        # To exponentiate by the first nibble, look it up in the table
        nib, nibbles = nibbles
        prod = lowPowers[nib]

        # For the rest, square nBitScan times, then multiply by
        # base^nibble
        while nibbles:
            nib, nibbles = nibbles
            for i in xrange(nBitScan):
                prod = (prod * prod) % modulus
            if nib: prod = (prod * lowPowers[nib]) % modulus

        #TREV - Added support for negative exponents
        if negativeResult:
            prodInv = invMod(prod, modulus)
            #Check to make sure the inverse is correct
            if (prod * prodInv) % modulus != 1:
                raise AssertionError()
            return prodInv
        return prod


#Pre-calculate a sieve of the ~100 primes < 1000:
def makeSieve(n):
    sieve = range(n)
    for count in range(2, int(math.sqrt(n))):
        if sieve[count] == 0:
            continue
        x = sieve[count] * 2
        while x < len(sieve):
            sieve[x] = 0
            x += sieve[count]
    sieve = [x for x in sieve[2:] if x]
    return sieve

sieve = makeSieve(1000)

def isPrime(n, iterations=5, display=False):
    #Trial division with sieve
    for x in sieve:
        if x >= n: return True
        if n % x == 0: return False
    #Passed trial division, proceed to Rabin-Miller
    #Rabin-Miller implemented per Ferguson & Schneier
    #Compute s, t for Rabin-Miller
    if display: print "*",
    s, t = n-1, 0
    while s % 2 == 0:
        s, t = s/2, t+1
    #Repeat Rabin-Miller x times
    a = 2 #Use 2 as a base for first iteration speedup, per HAC
    for count in range(iterations):
        v = powMod(a, s, n)
        if v==1:
            continue
        i = 0
        while v != n-1:
            if i == t-1:
                return False
            else:
                v, i = powMod(v, 2, n), i+1
        a = getRandomNumber(2, n)
    return True

def getRandomPrime(bits, display=False):
    if bits < 10:
        raise AssertionError()
    #The 1.5 ensures the 2 MSBs are set
    #Thus, when used for p,q in RSA, n will have its MSB set
    #
    #Since 30 is lcm(2,3,5), we'll set our test numbers to
    #29 % 30 and keep them there
    low = (2L ** (bits-1)) * 3/2
    high = 2L ** bits - 30
    p = getRandomNumber(low, high)
    p += 29 - (p % 30)
    while 1:
        if display: print ".",
        p += 30
        if p >= high:
            p = getRandomNumber(low, high)
            p += 29 - (p % 30)
        if isPrime(p, display=display):
            return p

#Unused at the moment...
def getRandomSafePrime(bits, display=False):
    if bits < 10:
        raise AssertionError()
    #The 1.5 ensures the 2 MSBs are set
    #Thus, when used for p,q in RSA, n will have its MSB set
    #
    #Since 30 is lcm(2,3,5), we'll set our test numbers to
    #29 % 30 and keep them there
    low = (2 ** (bits-2)) * 3/2
    high = (2 ** (bits-1)) - 30
    q = getRandomNumber(low, high)
    q += 29 - (q % 30)
    while 1:
        if display: print ".",
        q += 30
        if (q >= high):
            q = getRandomNumber(low, high)
            q += 29 - (q % 30)
        #Ideas from Tom Wu's SRP code
        #Do trial division on p and q before Rabin-Miller
        if isPrime(q, 0, display=display):
            p = (2 * q) + 1
            if isPrime(p, display=display):
                if isPrime(q, display=display):
                    return p



#!/usr/bin/python
#
# Copyright (C) 2007 Google Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""SpreadsheetsService extends the GDataService to streamline Google
Spreadsheets operations.

  SpreadsheetService: Provides methods to query feeds and manipulate items.
                      Extends GDataService.

  DictionaryToParamList: Function which converts a dictionary into a list of
                         URL arguments (represented as strings). This is a
                         utility function used in CRUD operations.
"""

__author__ = 'api.laurabeth@gmail.com (Laura Beth Lincoln)'


import gdata
import atom.service
import gdata.service
import gdata.spreadsheet
import atom


class Error(Exception):
  """Base class for exceptions in this module."""
  pass


class RequestError(Error):
  pass


class SpreadsheetsService(gdata.service.GDataService):
  """Client for the Google Spreadsheets service."""

  def __init__(self, email=None, password=None, source=None,
               server='spreadsheets.google.com', additional_headers=None,
               **kwargs):
    """Creates a client for the Google Spreadsheets service.

    Args:
      email: string (optional) The user's email address, used for
          authentication.
      password: string (optional) The user's password.
      source: string (optional) The name of the user's application.
      server: string (optional) The name of the server to which a connection
          will be opened. Default value: 'spreadsheets.google.com'.
      **kwargs: The other parameters to pass to gdata.service.GDataService
          constructor.
    """
    gdata.service.GDataService.__init__(
        self, email=email, password=password, service='wise', source=source,
        server=server, additional_headers=additional_headers, **kwargs)

  def GetSpreadsheetsFeed(self, key=None, query=None, visibility='private', 
      projection='full'):
    """Gets a spreadsheets feed or a specific entry if a key is defined
    Args:
      key: string (optional) The spreadsheet key defined in /ccc?key=
      query: DocumentQuery (optional) Query parameters
      
    Returns:
      If there is no key, then a SpreadsheetsSpreadsheetsFeed.
      If there is a key, then a SpreadsheetsSpreadsheet.
    """
    
    base_uri = 'https://%s/feeds/spreadsheets' % self.server
    uri = ('%s/%s/%s' 
           % (base_uri, visibility, projection))
    
    if key is not None:
      uri = '%s/%s' % (uri, key)
      
    if query != None:
      query.feed = base_uri
      query.visibility = visibility
      query.projection = projection
      uri = query.ToUri()

    if key:
      return self.Get(uri, 
          converter=gdata.spreadsheet.SpreadsheetsSpreadsheetFromString)
    else:
      return self.Get(uri,
          converter=gdata.spreadsheet.SpreadsheetsSpreadsheetsFeedFromString)
  
  def GetWorksheetsFeed(self, key, wksht_id=None, query=None, 
      visibility='private', projection='full'):
    """Gets a worksheets feed or a specific entry if a wksht is defined
    Args:
      key: string The spreadsheet key defined in /ccc?key=
      wksht_id: string (optional) The id for a specific worksheet entry
      query: DocumentQuery (optional) Query parameters
      
    Returns:
      If there is no wksht_id, then a SpreadsheetsWorksheetsFeed.
      If there is a wksht_id, then a SpreadsheetsWorksheet.
    """
    
    uri = ('https://%s/feeds/worksheets/%s/%s/%s' 
           % (self.server, key, visibility, projection))
    
    if wksht_id != None:
      uri = '%s/%s' % (uri, wksht_id)
      
    if query != None:
      query.feed = uri
      uri = query.ToUri()

    if wksht_id:
      return self.Get(uri, 
          converter=gdata.spreadsheet.SpreadsheetsWorksheetFromString)
    else:
      return self.Get(uri,
          converter=gdata.spreadsheet.SpreadsheetsWorksheetsFeedFromString)

  def AddWorksheet(self, title, row_count, col_count, key):
    """Creates a new worksheet in the desired spreadsheet.

    The new worksheet is appended to the end of the list of worksheets. The
    new worksheet will only have the available number of columns and cells 
    specified.

    Args:
      title: str The title which will be displayed in the list of worksheets.
      row_count: int or str The number of rows in the new worksheet.
      col_count: int or str The number of columns in the new worksheet.
      key: str The spreadsheet key to the spreadsheet to which the new 
          worksheet should be added. 

    Returns:
      A SpreadsheetsWorksheet if the new worksheet was created succesfully.  
    """
    new_worksheet = gdata.spreadsheet.SpreadsheetsWorksheet(
        title=atom.Title(text=title), 
        row_count=gdata.spreadsheet.RowCount(text=str(row_count)), 
        col_count=gdata.spreadsheet.ColCount(text=str(col_count)))
    return self.Post(new_worksheet, 
        'https://%s/feeds/worksheets/%s/private/full' % (self.server, key),
        converter=gdata.spreadsheet.SpreadsheetsWorksheetFromString)

  def UpdateWorksheet(self, worksheet_entry, url=None):
    """Changes the size and/or title of the desired worksheet.
    
    Args:
      worksheet_entry: SpreadsheetWorksheet The new contents of the 
          worksheet. 
      url: str (optional) The URL to which the edited worksheet entry should
          be sent. If the url is None, the edit URL from the worksheet will
          be used.

    Returns: 
      A SpreadsheetsWorksheet with the new information about the worksheet.
    """
    target_url = url or worksheet_entry.GetEditLink().href
    return self.Put(worksheet_entry, target_url, 
        converter=gdata.spreadsheet.SpreadsheetsWorksheetFromString)
    
  def DeleteWorksheet(self, worksheet_entry=None, url=None):
    """Removes the desired worksheet from the spreadsheet
    
    Args:
      worksheet_entry: SpreadsheetWorksheet (optional) The worksheet to
          be deleted. If this is none, then the DELETE reqest is sent to 
          the url specified in the url parameter.
      url: str (optaional) The URL to which the DELETE request should be
          sent. If left as None, the worksheet's edit URL is used.

    Returns:
      True if the worksheet was deleted successfully. 
    """
    if url:
      target_url = url
    else:
      target_url = worksheet_entry.GetEditLink().href
    return self.Delete(target_url)
  
  def GetCellsFeed(self, key, wksht_id='default', cell=None, query=None, 
      visibility='private', projection='full'):
    """Gets a cells feed or a specific entry if a cell is defined
    Args:
      key: string The spreadsheet key defined in /ccc?key=
      wksht_id: string The id for a specific worksheet entry
      cell: string (optional) The R1C1 address of the cell
      query: DocumentQuery (optional) Query parameters
      
    Returns:
      If there is no cell, then a SpreadsheetsCellsFeed.
      If there is a cell, then a SpreadsheetsCell.
    """
    
    uri = ('https://%s/feeds/cells/%s/%s/%s/%s' 
           % (self.server, key, wksht_id, visibility, projection))
    
    if cell != None:
      uri = '%s/%s' % (uri, cell)
      
    if query != None:
      query.feed = uri
      uri = query.ToUri()

    if cell:
      return self.Get(uri, 
          converter=gdata.spreadsheet.SpreadsheetsCellFromString)
    else:
      return self.Get(uri, 
          converter=gdata.spreadsheet.SpreadsheetsCellsFeedFromString)
  
  def GetListFeed(self, key, wksht_id='default', row_id=None, query=None,
      visibility='private', projection='full'):
    """Gets a list feed or a specific entry if a row_id is defined
    Args:
      key: string The spreadsheet key defined in /ccc?key=
      wksht_id: string The id for a specific worksheet entry
      row_id: string (optional) The row_id of a row in the list
      query: DocumentQuery (optional) Query parameters
      
    Returns:
      If there is no row_id, then a SpreadsheetsListFeed.
      If there is a row_id, then a SpreadsheetsList.
    """
    
    uri = ('https://%s/feeds/list/%s/%s/%s/%s' 
           % (self.server, key, wksht_id, visibility, projection))

    if row_id is not None:
      uri = '%s/%s' % (uri, row_id)
      
    if query is not None:
      query.feed = uri
      uri = query.ToUri()

    if row_id:
      return self.Get(uri, 
          converter=gdata.spreadsheet.SpreadsheetsListFromString)
    else:
      return self.Get(uri, 
          converter=gdata.spreadsheet.SpreadsheetsListFeedFromString)
    
  def UpdateCell(self, row, col, inputValue, key, wksht_id='default'):
    """Updates an existing cell.
    
    Args:
      row: int The row the cell to be editted is in
      col: int The column the cell to be editted is in
      inputValue: str the new value of the cell
      key: str The key of the spreadsheet in which this cell resides.
      wksht_id: str The ID of the worksheet which holds this cell.
      
    Returns:
      The updated cell entry
    """
    row = str(row)
    col = str(col)
    # make the new cell
    new_cell = gdata.spreadsheet.Cell(row=row, col=col, inputValue=inputValue)
    # get the edit uri and PUT
    cell = 'R%sC%s' % (row, col)
    entry = self.GetCellsFeed(key, wksht_id, cell)
    for a_link in entry.link:
      if a_link.rel == 'edit':
        entry.cell = new_cell
        return self.Put(entry, a_link.href, 
            converter=gdata.spreadsheet.SpreadsheetsCellFromString)

  def _GenerateCellsBatchUrl(self, spreadsheet_key, worksheet_id):
    return ('https://spreadsheets.google.com/feeds/cells/%s/%s/'
            'private/full/batch' % (spreadsheet_key, worksheet_id))

  def ExecuteBatch(self, batch_feed, url=None, spreadsheet_key=None, 
      worksheet_id=None,
      converter=gdata.spreadsheet.SpreadsheetsCellsFeedFromString):
    """Sends a batch request feed to the server.

    The batch request needs to be sent to the batch URL for a particular 
    worksheet. You can specify the worksheet by providing the spreadsheet_key
    and worksheet_id, or by sending the URL from the cells feed's batch link.

    Args:
      batch_feed: gdata.spreadsheet.SpreadsheetsCellFeed A feed containing 
          BatchEntry elements which contain the desired CRUD operation and 
          any necessary data to modify a cell.
      url: str (optional) The batch URL for the cells feed to which these 
          changes should be applied. This can be found by calling 
          cells_feed.GetBatchLink().href.
      spreadsheet_key: str (optional) Used to generate the batch request URL
          if the url argument is None. If using the spreadsheet key to 
          generate the URL, the worksheet id is also required.
      worksheet_id: str (optional) Used if the url is not provided, it is 
          oart of the batch feed target URL. This is used with the spreadsheet
          key.
      converter: Function (optional) Function to be executed on the server's
          response. This function should take one string as a parameter. The
          default value is SpreadsheetsCellsFeedFromString which will turn the result
          into a gdata.spreadsheet.SpreadsheetsCellsFeed object.

    Returns:
      A gdata.BatchFeed containing the results.
    """

    if url is None:
      url = self._GenerateCellsBatchUrl(spreadsheet_key, worksheet_id)
    return self.Post(batch_feed, url, converter=converter)
    
  def InsertRow(self, row_data, key, wksht_id='default'):
    """Inserts a new row with the provided data
    
    Args:
      uri: string The post uri of the list feed
      row_data: dict A dictionary of column header to row data
    
    Returns:
      The inserted row
    """
    new_entry = gdata.spreadsheet.SpreadsheetsList()
    for k, v in row_data.iteritems():
      new_custom = gdata.spreadsheet.Custom()
      new_custom.column = k
      new_custom.text = v
      new_entry.custom[new_custom.column] = new_custom
    # Generate the post URL for the worksheet which will receive the new entry.
    post_url = 'https://spreadsheets.google.com/feeds/list/%s/%s/private/full'%(
        key, wksht_id) 
    return self.Post(new_entry, post_url, 
        converter=gdata.spreadsheet.SpreadsheetsListFromString)
    
  def UpdateRow(self, entry, new_row_data):
    """Updates a row with the provided data

    If you want to add additional information to a row, it is often
    easier to change the values in entry.custom, then use the Put
    method instead of UpdateRow. This UpdateRow method will replace
    the contents of the row with new_row_data - it will change all columns
    not just the columns specified in the new_row_data dict.
    
    Args:
      entry: gdata.spreadsheet.SpreadsheetsList The entry to be updated
      new_row_data: dict A dictionary of column header to row data
      
    Returns:
      The updated row
    """
    entry.custom = {}
    for k, v in new_row_data.iteritems():
      new_custom = gdata.spreadsheet.Custom()
      new_custom.column = k
      new_custom.text = v
      entry.custom[k] = new_custom
    for a_link in entry.link:
      if a_link.rel == 'edit':
        return self.Put(entry, a_link.href, 
            converter=gdata.spreadsheet.SpreadsheetsListFromString)
        
  def DeleteRow(self, entry):
    """Deletes a row, the provided entry
    
    Args:
      entry: gdata.spreadsheet.SpreadsheetsList The row to be deleted
    
    Returns:
      The delete response
    """
    for a_link in entry.link:
      if a_link.rel == 'edit':
        return self.Delete(a_link.href)


class DocumentQuery(gdata.service.Query):

  def _GetTitleQuery(self):
    return self['title']

  def _SetTitleQuery(self, document_query):
    self['title'] = document_query
    
  title = property(_GetTitleQuery, _SetTitleQuery,
      doc="""The title query parameter""")

  def _GetTitleExactQuery(self):
    return self['title-exact']

  def _SetTitleExactQuery(self, document_query):
    self['title-exact'] = document_query
    
  title_exact = property(_GetTitleExactQuery, _SetTitleExactQuery,
      doc="""The title-exact query parameter""")
 
 
class CellQuery(gdata.service.Query):

  def _GetMinRowQuery(self):
    return self['min-row']

  def _SetMinRowQuery(self, cell_query):
    self['min-row'] = cell_query
    
  min_row = property(_GetMinRowQuery, _SetMinRowQuery,
      doc="""The min-row query parameter""")

  def _GetMaxRowQuery(self):
    return self['max-row']

  def _SetMaxRowQuery(self, cell_query):
    self['max-row'] = cell_query
    
  max_row = property(_GetMaxRowQuery, _SetMaxRowQuery,
      doc="""The max-row query parameter""")
      
  def _GetMinColQuery(self):
    return self['min-col']

  def _SetMinColQuery(self, cell_query):
    self['min-col'] = cell_query
    
  min_col = property(_GetMinColQuery, _SetMinColQuery,
      doc="""The min-col query parameter""")
      
  def _GetMaxColQuery(self):
    return self['max-col']

  def _SetMaxColQuery(self, cell_query):
    self['max-col'] = cell_query
    
  max_col = property(_GetMaxColQuery, _SetMaxColQuery,
      doc="""The max-col query parameter""")
      
  def _GetRangeQuery(self):
    return self['range']

  def _SetRangeQuery(self, cell_query):
    self['range'] = cell_query
    
  range = property(_GetRangeQuery, _SetRangeQuery,
      doc="""The range query parameter""")
      
  def _GetReturnEmptyQuery(self):
    return self['return-empty']

  def _SetReturnEmptyQuery(self, cell_query):
    self['return-empty'] = cell_query
    
  return_empty = property(_GetReturnEmptyQuery, _SetReturnEmptyQuery,
      doc="""The return-empty query parameter""")
 
 
class ListQuery(gdata.service.Query):

  def _GetSpreadsheetQuery(self):
    return self['sq']

  def _SetSpreadsheetQuery(self, list_query):
    self['sq'] = list_query
    
  sq = property(_GetSpreadsheetQuery, _SetSpreadsheetQuery,
      doc="""The sq query parameter""")
      
  def _GetOrderByQuery(self):
    return self['orderby']

  def _SetOrderByQuery(self, list_query):
    self['orderby'] = list_query
    
  orderby = property(_GetOrderByQuery, _SetOrderByQuery,
      doc="""The orderby query parameter""")
      
  def _GetReverseQuery(self):
    return self['reverse']

  def _SetReverseQuery(self, list_query):
    self['reverse'] = list_query
    
  reverse = property(_GetReverseQuery, _SetReverseQuery,
      doc="""The reverse query parameter""")
